/*
   +----------------------------------------------------------------------+
   | ayTemplates version 0.02-dev                                         |
   +----------------------------------------------------------------------+
   | Copyright (c) 2001, 2002 David C A Croft                             |
   +----------------------------------------------------------------------+
   | This source file is subject to version 2.02 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
   | available at through the world-wide-web at                           |
   | http://www.php.net/license/2_02.txt.                                 |
   | If you did not receive a copy of the PHP license and are unable to   |
   | obtain it through the world-wide-web, please send a note to          |
   | license@php.net so we can mail you a copy immediately.               |
   +----------------------------------------------------------------------+
   | Author: David Croft <david@infotrek.co.uk>                           |
   +----------------------------------------------------------------------+
 */

#define TPL_VERSION "0.03"

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "php_ini.h"
#include "php_tpl.h"

/* If you declare any globals in php_tpl.h uncomment this:
ZEND_DECLARE_MODULE_GLOBALS(tpl)
*/

/* True global resources - no need for thread safety here */
static int le_tpl;

/* Every user visible function must have an entry in tpl_functions[].
*/
function_entry tpl_functions[] = {
	PHP_FE(tpl_parse,	first_arg_force_ref)
	{NULL, NULL, NULL}
};

zend_module_entry tpl_module_entry = {
#if ZEND_MODULE_API_NO >= 20010901
    STANDARD_MODULE_HEADER,
#endif
	"tpl",
	tpl_functions,
	NULL, // PHP_MINIT(tpl),
	NULL, // PHP_MSHUTDOWN(tpl),
	NULL, // PHP_RINIT(tpl),		/* Replace with NULL if there's nothing to do at request start */
	NULL, // PHP_RSHUTDOWN(tpl),	/* Replace with NULL if there's nothing to do at request end */
	PHP_MINFO(tpl),
#if ZEND_MODULE_API_NO >= 20010901
    TPL_VERSION,          /* extension version number (string) */
#endif
	STANDARD_MODULE_PROPERTIES
};

#ifdef COMPILE_DL_TPL
ZEND_GET_MODULE(tpl)
#endif

/* Remove comments and fill if you need to have entries in php.ini
PHP_INI_BEGIN()
PHP_INI_END()
*/

PHP_MINIT_FUNCTION(tpl)
{
/* Remove comments if you have entries in php.ini
	REGISTER_INI_ENTRIES();
*/
	return SUCCESS;
}

PHP_MSHUTDOWN_FUNCTION(tpl)
{
/* Remove comments if you have entries in php.ini
	UNREGISTER_INI_ENTRIES();
*/
	return SUCCESS;
}

/* Remove if there's nothing to do at request start */
PHP_RINIT_FUNCTION(tpl)
{
	return SUCCESS;
}

/* Remove if there's nothing to do at request end */
PHP_RSHUTDOWN_FUNCTION(tpl)
{
	return SUCCESS;
}

PHP_MINFO_FUNCTION(tpl)
{
	php_info_print_table_start();
	php_info_print_table_header(2, "tpl support", "enabled - version " TPL_VERSION);
	php_info_print_table_end();

	/* Remove comments if you have entries in php.ini
	DISPLAY_INI_ENTRIES();
	*/
}

void append_string_length(zval *target, char *src, int srclen)
{
	/* target MUST be an ealloced string - assumed */

	int length = Z_STRLEN_P(target) + srclen;

	Z_STRVAL_P(target) = (char *) erealloc(Z_STRVAL_P(target), length + 1);

	memcpy(Z_STRVAL_P(target) + Z_STRLEN_P(target), src, srclen);
	Z_STRVAL_P(target)[length] = 0;

	Z_STRLEN_P(target) = length;
}

/* {{{ proto string tpl_parse(string template, array vars [, string callback_function [, object callback_object]])
    */
PHP_FUNCTION(tpl_parse)
{
	zval **template_arg, **vars_arg, **vars_entry;
	zval **callback_function_arg = NULL;
	zval **callback_object_arg = NULL;
	zval *callback_retval;
	zval *variable;
	zval tempval;
	int argc, callback_result;

	char *copy_from, *copy_to, *template_end, *locate_close;

	argc = ZEND_NUM_ARGS();
	if (argc < 2 || argc > 4 || zend_get_parameters_ex(argc, &template_arg, &vars_arg, &callback_function_arg, &callback_object_arg) == FAILURE) {
		WRONG_PARAM_COUNT;
	}

	switch (argc) {
		case 4:
			if ((*callback_object_arg)->type != IS_OBJECT) {
				php_error(E_WARNING, "arg 4 to tpl_parse should be an object");
				RETURN_FALSE;
			}
			/* Fall-through. */

		case 3:
			convert_to_string_ex(callback_function_arg);
			ELS_FETCH();
			/* Fall-through. */

		case 2:
			convert_to_array_ex(vars_arg);
			convert_to_string_ex(template_arg);
			break;

		EMPTY_SWITCH_DEFAULT_CASE();
	}

	return_value->type = IS_STRING;
	return_value->value.str.len = 0;
	return_value->value.str.val = emalloc(1);
	return_value->value.str.val[0] = 0;

	copy_from = Z_STRVAL_PP(template_arg);
	template_end = Z_STRVAL_PP(template_arg) + Z_STRLEN_PP(template_arg);

	for (copy_to = copy_from; copy_to < template_end; copy_to ++) {
		if (*copy_to != '{') {
			continue;
		}

		if (*(copy_to + 1) == '{') {
			/* {{ is treated as a single { */
			
			if (copy_to - copy_from > 0) {
				/* Output pending text */

				append_string_length(return_value, copy_from, copy_to-copy_from);
			}
			copy_to ++;
			copy_from = copy_to;
			continue;
		}

		locate_close = copy_to + 1;

		for (locate_close = copy_to + 1; locate_close < template_end; locate_close ++ ) {
			/* Looking for the close bracket */

			if (*locate_close < 33) {
				/* Space or control character means this bracket isn't for us */
				break;
			}
			else if (*locate_close == '}') {
				/* Found the terminating bracket. */

				if (*locate_close == '}' && locate_close == copy_to + 1) {
					/* {} is sometimes used in javascript and is not parsed */
					break;
				}

				if (copy_to - copy_from > 0) {
					/* Output pending text */

					append_string_length(return_value, copy_from, copy_to-copy_from);
				}

				/* Construct a zval string with the variable name */

				MAKE_STD_ZVAL(variable);
				Z_TYPE_P(variable) = IS_STRING;

				Z_STRVAL_P(variable) = emalloc(locate_close - copy_to); /* +1 -1 */
				memcpy(Z_STRVAL_P(variable), copy_to+1, (locate_close - copy_to) - 1);
				Z_STRVAL_P(variable)[(locate_close - copy_to) - 1] = 0;
				Z_STRLEN_P(variable) = (locate_close - copy_to) - 1;


				if (zend_hash_find(HASH_OF(*vars_arg), Z_STRVAL_P(variable), Z_STRLEN_P(variable)+1, (void **) &vars_entry) == SUCCESS) {

					if (Z_TYPE_PP(vars_entry) == IS_STRING) {
						append_string_length(return_value, Z_STRVAL_PP(vars_entry), Z_STRLEN_PP(vars_entry));
					}
					else {
						/* We can't touch the vars_entry zval cos that would alter the original hash */
						/* so we must copy before convert_to_string */

						tempval = **vars_entry;
						zval_copy_ctor(&tempval);
						convert_to_string(&tempval);

						append_string_length(return_value, Z_STRVAL(tempval), Z_STRLEN(tempval));

						zval_dtor(&tempval);
					}
				}
				else if (callback_function_arg) {
					/* Call back function, if it exists */

					MAKE_STD_ZVAL(callback_retval);
					callback_retval->type = IS_NULL;
					callback_retval->value.lval = 0;

					callback_result = call_user_function(EG(function_table), callback_object_arg, *callback_function_arg, callback_retval, 1, &variable);

					if (callback_result == FAILURE) {
						if (Z_TYPE_PP(callback_function_arg) == IS_STRING) {
							php_error(E_WARNING, "Unable to call handler %s()", Z_STRVAL_PP(callback_function_arg));   
						}
						else {
							php_error(E_WARNING, "Unable to call handler");
						}
					}

					if (callback_result == FAILURE ||
						(callback_retval->type == IS_BOOL && callback_retval->value.lval == 0)) {

						/* If the user function returns false, they want us to abort */
						/* We assume they have reported the error if necessary so are silent */

						zval_dtor(callback_retval);
						efree(callback_retval);
						zval_dtor(return_value); /* Kill our string-in-progress */
						RETURN_FALSE;
					}

					/* Any other result gets converted to a string and appended */

					convert_to_string(callback_retval);

					append_string_length(return_value, Z_STRVAL_P(callback_retval), Z_STRLEN_P(callback_retval));

				}

				zval_dtor(variable);
				efree(variable);

				/* Restart the pending text marker after the current var */

				copy_from = locate_close + 1;
			}
		}
	}

	if (copy_to - copy_from > 0) {
		/* Output any remaining text */
		append_string_length(return_value, copy_from, copy_to-copy_from);
	}
}
/* }}} */



/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * indent-tabs-mode: t
 * End:
 */
